Package: uaparserjs
Type: Package
Title: Parse 'User-Agent' Strings
Version: 0.3.8
Date: 2026-05-09
Authors@R: c(
    person("Bob", "Rudis", , "bob@rud.is", role = c("aut", "cph")),
    person("Greg", "Hunt", , "greg@firmansyah.com", role = c("aut", "cre", "cph")),
    person("Lindsey", "Simon", role = "aut", comment = "uap-core"),
    person("Tobie", "Langel", role = "aut", comment = "uap-core"),
    person("Colman", "Humphrey", role = "ctb"))
Description: Despite there being a section in RFC 7231
    <https://datatracker.ietf.org/doc/html/rfc7231#section-5.5.3> defining a suggested
    structure for 'User-Agent' headers this data is notoriously difficult
    to parse consistently. Tools are provided that will take in user agent
    strings and return structured R objects. This is a 'V8'-backed package
    based on the 'ua-parser' project <https://github.com/ua-parser>.
URL: https://github.com/gregfrog/uaparserjs
BugReports: https://github.com/gregfrog/uaparserjs/issues
License: Apache License 2.0 | file LICENSE
Suggests: tinytest, yaml
Depends: R (>= 3.5.0)
Encoding: UTF-8
Imports: progress, V8
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-05-09 06:49:49 UTC; greg
Author: Bob Rudis [aut, cph],
  Greg Hunt [aut, cre, cph],
  Lindsey Simon [aut] (uap-core),
  Tobie Langel [aut] (uap-core),
  Colman Humphrey [ctb]
Maintainer: Greg Hunt <greg@firmansyah.com>
Repository: CRAN
Date/Publication: 2026-05-09 14:40:12 UTC
Built: R 4.6.0; ; 2026-05-09 17:51:16 UTC; unix
