Package: uaparserjs
Type: Package
Title: Parse 'User-Agent' Strings
Version: 0.4.0
Date: 2026-07-02
Authors@R: c(
    person("Bob", "Rudis", , "bob@rud.is", role = c("aut", "cph")),
    person("Greg", "Hunt", , "greg@firmansyah.com", role = c("aut", "cre", "cph")),
    person("Lindsey", "Simon", role = "aut", comment = "uap-core"),
    person("Tobie", "Langel", role = "aut", comment = "uap-core"),
    person("Colman", "Humphrey", role = "ctb"),
    person("Jim", "Vine", role = "ctb"))
Description: 
    Parses HTTP user agent strings and returns user agent, device and OS information.  This is a 'V8'-backed package
    based on the 'ua-parser' project <https://github.com/ua-parser>.
URL: https://github.com/gregfrog/uaparserjs
BugReports: https://github.com/gregfrog/uaparserjs/issues
License: Apache License 2.0 | file LICENSE
Suggests: tinytest, yaml, tibble
Depends: R (>= 3.5.0)
Encoding: UTF-8
Imports: progress, V8
NeedsCompilation: no
Packaged: 2026-07-01 22:19:07 UTC; greg
Author: Bob Rudis [aut, cph],
  Greg Hunt [aut, cre, cph],
  Lindsey Simon [aut] (uap-core),
  Tobie Langel [aut] (uap-core),
  Colman Humphrey [ctb],
  Jim Vine [ctb]
Maintainer: Greg Hunt <greg@firmansyah.com>
Repository: CRAN
Date/Publication: 2026-07-02 21:50:02 UTC
Built: R 4.6.1; ; 2026-07-02 22:40:42 UTC; unix
